/////////////////////////////////////////////////////////////////////////////
//  File:       BinTree.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
// parts of code by Zafir Anjum (Converting DDB to DIB )
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
// Version 1.0.0.1: Export to emf/wmf is now supported. Added a new method  
//					BOOL WriteWMF(CString strFile). Suggested by Roman Nurik
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bintree.h"

/////////////////////////////////////////////////////////////////////////////
IMPLEMENT_SERIAL(CBinTree, CObject, 1)

CBinTree::CBinTree()
{
	m_pRootNode=NULL;
	m_pSelectNode=NULL;
	m_dwSelectType=NULL;
	m_strName="";
}

CBinTree::~CBinTree()
{
	// clean up the memory
	if(m_pRootNode)	
		delete m_pRootNode;
}

// return the root node of the tree
CNode* CBinTree::GetRootNode()
{
	return m_pRootNode;
}

// sets the root node of the tree
void CBinTree::SetRootNode(CNode* pNode)
{
	m_pRootNode=pNode;
}

// return the name of the tree
CString CBinTree::GetName()
{
	return m_strName;
}

// set the name of the tree
void CBinTree::SetName(CString strName)
{
	m_strName=strName;
}

// create a tree with node of type strType as root
CNode* CBinTree::CreateTree(CString strType)
{
	m_pRootNode=CreateNode(strType);
	return m_pRootNode;
}

// reset the complete tree and delete all nodes
void CBinTree::ResetTree()
{
	DeleteContents();
}

// create a node of a given type
CNode* CBinTree::CreateNode(CString strType)
{
	CNode* pNode=NULL;

	if (strType == NT_STANDARD)
	{
		pNode=(CNode*) new CNode();
		pNode->m_dwEditMode &= NE_ALLOWEDIT;
	}
	else if (strType == NT_DIVISION)	pNode=(CNode*) new CDivisionNode();
	else if (strType == NT_NTHROOT)		pNode=(CNode*) new CNthRootNode();
	else if (strType == NT_ROOT)		pNode=(CNode*) new CSquareRootNode();
	else if (strType == NT_POWERTO)		pNode=(CNode*) new CPowerToNode();
	else if (strType == NT_PLACEHOLDER)	pNode=(CNode*) new CPlaceHolderNode();
	else if (strType == NT_BRACE)		pNode=(CNode*) new CBraceNode();
	else if (strType == NT_EQUATION)	pNode=(CNode*) new CEquationNode();
	else if (strType == NT_PLUS)		pNode=(CNode*) new CPlusNode();
	else if (strType == NT_MINUS)		pNode=(CNode*) new CMinusNode();
	else if (strType == NT_TIMES)		pNode=(CNode*) new CTimesNode();
	else if (strType == NT_NUMBER)		pNode=(CNode*) new CNumberNode();
	else if (strType == NT_PARTDERIVE)	pNode=(CNode*) new CPartialDerivativeNode();
	else if (strType == NT_DERIVE)		pNode=(CNode*) new CDerivativeNode();
	else if (strType == NT_SUM)			pNode=(CNode*) new CSumNode();
	else if (strType == NT_PROD)		pNode=(CNode*) new CProdNode();
	else if (strType == NT_RANGE)		pNode=(CNode*) new CRangeNode();
	else if (strType == NT_INTEGRAL)	pNode=(CNode*) new CIntegralNode();
	else if (strType == NT_INTEGRAND)	pNode=(CNode*) new CIntegrandNode();
	else if (strType == NT_FUNC)		pNode=(CNode*) new CFuncNode();
	else if (strType == NT_USERFUNC)	pNode=(CNode*) new CUserFuncNode();
	else if (strType == NT_EXTFUNC)		pNode=(CNode*) new CExtFuncNode();
	else if (strType == NT_POISON)		pNode=(CNode*) new CPoisonNode();
	else if (strType == NT_INDEX)		pNode=(CNode*) new CIndexNode();
	else if (strType == NT_VECTOR)		pNode=(CNode*) new CVectorNode();
	else if (strType == NT_CONSTANT)	pNode=(CNode*) new CConstantNode();
	else if (strType == NT_VARIABLE)	pNode=(CNode*) new CVariableNode();
	else if (strType == NT_LIMES)		pNode=(CNode*) new CLimesNode();
	else if (strType == NT_OPERATOR)	pNode=(CNode*) new COperatorNode();
	else if (strType == NT_ARROWS)		pNode=(CNode*) new CArrowNode();
	else if (strType == NT_BINOMIAL)	pNode=(CNode*) new CBinomialNode();
	else if (strType == NT_MATRIX)		pNode=(CNode*) new CMatrixNode();
	else if (strType == NT_LINE)		pNode=(CNode*) new CLineNode();
	else if (strType == NT_ELEMENT)		pNode=(CNode*) new CElementNode();
	else if (strType == NT_NABLA)		pNode=(CNode*) new CNablaNode();
	else if (strType == NT_LAPLACE)		pNode=(CNode*) new CLaplaceNode();
	else if (strType == NT_INFINITY)	pNode=(CNode*) new CInfinityNode();
	else if (strType == NT_PLANCK)		pNode=(CNode*) new CPlanckNode();
	else if (strType == NT_LAMBDA)		pNode=(CNode*) new CLambdaNode();
	else if (strType == NT_CROSS)		pNode=(CNode*) new CCrossNode();
	else if (strType == NT_LABEL)		pNode=(CNode*) new CLabelNode();
	else pNode=(CNode*) new CNode();

	if(pNode)
	{
		pNode->m_strNodeType=strType;
		pNode->FormatNode(GetFormat());
	}
	
	return pNode;
}

// draw the tree (inorder)
void CBinTree::DrawTree(CDC* pDC, CRect rect)
{
	pDC->SetBkMode(TRANSPARENT);

	if(m_pRootNode)
		m_pRootNode->DrawNode(pDC, rect, m_pSelectNode, m_dwSelectType);
}

// returns the node and the drawing-rectangle of the node of a given mouse coordinate 
CNode* CBinTree::GetNodeFromPoint(CDC* pDC,CRect& resultRect, CPoint point)
{
	if(m_pRootNode)
	{
		CNode* pNode=m_pRootNode->GetNodeFromPoint(pDC, resultRect, point);
		return pNode;
	}
	else
		return NULL;
}

// reformat tree with a given Format
// if pFormat=NULL use the Default Treeformat (global GetFormat() method)
void CBinTree::ReformatNodes(CBinTreeFormat* pFormat)
{
	if(m_pRootNode)
	{
		if(pFormat)
			m_pRootNode->FormatNode(pFormat);
		else
			m_pRootNode->FormatNode(GetFormat());
	}
}

// select a node with a given selectiontype 
// NS_NODE (only the node)
// NS_SUBTREE (complete subtree)
void CBinTree::SelectNode(CNode* pNode, DWORD dwSelectType)
{
	m_pSelectNode=pNode;
	m_dwSelectType=dwSelectType;
}

// return the currently selected node
CNode* CBinTree::GetSelectedNode()
{
	return m_pSelectNode;
}

// return the currently selected nodeselection-type
// NS_NODE (only the node)
// NS_SUBTREE (complete subtree)
DWORD CBinTree::GetNodeSelectionType()
{
	return m_dwSelectType;
}

// set nodeselection-type
// NS_NODE (only the node)
// NS_SUBTREE (complete subtree)
void CBinTree::SetNodeSelectionType(DWORD dwSelectType)
{
	m_dwSelectType=dwSelectType;
}

//delete the node pNode
void CBinTree::DeleteNode(CNode* pNode)
{
	if(pNode)
		delete pNode;
}

const void CBinTree::operator=(const CBinTree& BinTree)
{
    CBinTree& rBinTree= (CBinTree&) BinTree;
	DeleteContents();
	
	m_pRootNode=CloneNode(rBinTree.GetRootNode(), TRUE);
	m_strName=rBinTree.GetName();
	ReformatNodes();
}

//clone the node pNode 
CNode* CBinTree::CloneNode(CNode* pNode, BOOL bCloneSubTree)
{
	CNode* pLeftChild=NULL;
	CNode* pRightChild=NULL;
	CNode* pCloneNode=NULL;

	if(!pNode)
		return NULL;
	
	pCloneNode=CreateNode(pNode->GetNodeType());
	(*pCloneNode)=(*pNode);

	if(bCloneSubTree)
	{
		if(pNode->GetLeftChild())
		{
			// if cunstructor of nodes adds children by default
			// delete them at first
			if(pCloneNode->GetLeftChild())
				DeleteNode(pCloneNode->GetLeftChild());
			pCloneNode->SetLeftChild(CloneNode(pNode->GetLeftChild()));
		}
		if(pNode->GetRightChild())
		{
			// if cunstructor of nodes adds children by default
			// delete them at first
			if(pCloneNode->GetRightChild())
				DeleteNode(pCloneNode->GetRightChild());
			pCloneNode->SetRightChild(CloneNode(pNode->GetRightChild()));
		}
	}
	return pCloneNode;
}

// search preoder in the tree for the parent of pNode
CNode* CBinTree::GetParent(CNode* pNode)
{
	if(m_pRootNode)
		return m_pRootNode->FindParentPraeorder(pNode);
	else
		return NULL;
}

// replace a single node (pDestNode) with a node of a given type (strType)
CNode* CBinTree::ReplaceNode(CNode* pDestNode, CString strType)
{
	CNode* pParent=GetParent(pDestNode);
	
	CNode* pLeftChild=NULL;
	CNode* pRightChild=NULL;

	if(pDestNode)
	{
		pLeftChild=pDestNode->GetLeftChild();
		pRightChild=pDestNode->GetRightChild();

		pDestNode->SetLeftChild(NULL);
		pDestNode->SetRightChild(NULL);
	}

	CNode* pNewNode=CreateNode(strType);
	pNewNode->SetLeftChild(pLeftChild);
	pNewNode->SetRightChild(pRightChild);
	
	if(pParent->GetLeftChild()==pDestNode)
		pParent->SetLeftChild(pNewNode);
	else if(pParent->GetRightChild()==pDestNode)
		pParent->SetRightChild(pNewNode);
	if(m_pSelectNode==pDestNode)
		m_pSelectNode=pNewNode;

	delete pDestNode;

	return pNewNode;
}

// replace pDestNode with pSrcNode 
void CBinTree::ReplaceNode(CNode* pDestNode, CNode* pSrcNode)
{
	CNode* pParent=GetParent(pDestNode);
	
	if(pParent->GetLeftChild()==pDestNode)
		pParent->SetLeftChild(pSrcNode);
	else if(pParent->GetRightChild()==pDestNode)
		pParent->SetRightChild(pSrcNode);
	
	if(m_pSelectNode==pDestNode)
		m_pSelectNode=pSrcNode;

	delete pDestNode;
}

// replaces a complete subtree(pDestNode) with a node of a given type(strType)
CNode* CBinTree::ReplaceSubTree(CNode* pDestNode, CString strType)
{
	CNode* pParent=GetParent(pDestNode);
	if(!pParent)
		return NULL;

	CNode* pNewNode=CreateNode(strType);
	
	if(pParent->GetLeftChild()==pDestNode)
		pParent->SetLeftChild(pNewNode);
	else if(pParent->GetRightChild()==pDestNode)
		pParent->SetRightChild(pNewNode);

	if(m_pSelectNode==pDestNode)
		m_pSelectNode=pNewNode;

	delete pDestNode;

	return pNewNode;
}

// insert a new node of a given type (strType) between the nodes pParent and pChild
// place the node pChild into the new position ()
// NI_CHILDLEFT (make pChild to the left child of pNewNode)
// NI_CHILDRIGHT (make pChild to the right child of pNewNode)
CNode* CBinTree::InsertNode(CNode* pParent, CNode* pChild, CString strType, DWORD dwInsert)
{
	CNode* pNewNode=CreateNode(strType);

	if(pParent->GetLeftChild()==pChild)
		pParent->SetLeftChild(pNewNode);
	else if(pParent->GetRightChild()==pChild)
		pParent->SetRightChild(pNewNode);
	
	
	if(dwInsert==NI_CHILDLEFT)
	{
		DeleteNode(pNewNode->GetLeftChild());
		pNewNode->SetLeftChild(pChild);
	}
	else if(dwInsert==NI_CHILDRIGHT)
	{
		DeleteNode(pNewNode->GetRightChild());
		pNewNode->SetRightChild(pChild);
	}

	return pNewNode;
}

// Serialize the tree (used by undo/redo)
void CBinTree::Serialize(CArchive& ar)
{
	m_pSelectNode=NULL;
	if (ar.IsStoring())
	{
		ar << m_strName;
		if(GetRootNode())
		{
			ar << GetRootNode()->m_strNodeType;
			GetRootNode()->Serialize(ar, this);
		}
		else
			ar << 0;
	}
	else
	{
		ar >> m_strName;
		CString strType;
		ar >> strType;
		if(strType!="")
		{
			ResetTree();
			CreateTree(strType);
			GetRootNode()->Serialize(ar, this);
		}
	}
}

// Delete the contents of the tree needed by undo/redo
// same behaviour as ResetTree
void CBinTree::DeleteContents()
{
	if(GetRootNode())
	{
		delete m_pRootNode;
		m_pRootNode=NULL;
	}
}

// returns the destination rectangle of the complete formula
CRect CBinTree::GetRect(CDC* pDC)
{
	if(GetRootNode())
	{
		return m_pRootNode->GetRect(pDC);
	}
	else
		return CRect(0,0,0,0);
}

// DDBToDIB		- Creates a DIB from a DDB
// bitmap		- Device dependent bitmap
// dwCompression	- Type of compression - see BITMAPINFOHEADER
// pPal			- Logical palette
HANDLE DDBToDIB( CBitmap& bitmap, DWORD dwCompression, CPalette* pPal ) 
{
// by Zafir Anjum
	BITMAP			bm;
	BITMAPINFOHEADER	bi;
	LPBITMAPINFOHEADER 	lpbi;
	DWORD			dwLen;
	HANDLE			hDIB;
	HANDLE			handle;
	HDC 			hDC;
	HPALETTE		hPal;


	ASSERT( bitmap.GetSafeHandle() );

	// The function has no arg for bitfields
	if( dwCompression == BI_BITFIELDS )
		return NULL;

	// If a palette has not been supplied use defaul palette
	if(pPal==NULL)
		hPal = (HPALETTE) GetStockObject(DEFAULT_PALETTE);
	else
	{
		hPal = (HPALETTE) pPal->GetSafeHandle();
		if (hPal==NULL)
			hPal = (HPALETTE) GetStockObject(DEFAULT_PALETTE);
	}

	// Get bitmap information
	bitmap.GetObject(sizeof(bm),(LPSTR)&bm);

	// Initialize the bitmapinfoheader
	bi.biSize		= sizeof(BITMAPINFOHEADER);
	bi.biWidth		= bm.bmWidth;
	bi.biHeight 		= bm.bmHeight;
	bi.biPlanes 		= 1;
	bi.biBitCount		= bm.bmPlanes * bm.bmBitsPixel;
	bi.biCompression	= dwCompression;
	bi.biSizeImage		= 0;
	bi.biXPelsPerMeter	= 0;
	bi.biYPelsPerMeter	= 0;
	bi.biClrUsed		= 0;
	bi.biClrImportant	= 0;

	// Compute the size of the  infoheader and the color table
	int nColors = (1 << bi.biBitCount);
	if( nColors > 256 ) 
		nColors = 0;
	dwLen  = bi.biSize + nColors * sizeof(RGBQUAD);

	// We need a device context to get the DIB from
	hDC = GetDC(NULL);
	hPal = SelectPalette(hDC,hPal,FALSE);
	RealizePalette(hDC);

	// Allocate enough memory to hold bitmapinfoheader and color table
	hDIB = GlobalAlloc(GMEM_FIXED,dwLen);

	if (!hDIB){
		SelectPalette(hDC,hPal,FALSE);
		ReleaseDC(NULL,hDC);
		return NULL;
	}

	lpbi = (LPBITMAPINFOHEADER)hDIB;

	*lpbi = bi;

	// Call GetDIBits with a NULL lpBits param, so the device driver 
	// will calculate the biSizeImage field 
	GetDIBits(hDC, (HBITMAP)bitmap.GetSafeHandle(), 0L, (DWORD)bi.biHeight,
			(LPBYTE)NULL, (LPBITMAPINFO)lpbi, (DWORD)DIB_RGB_COLORS);

	bi = *lpbi;

	// If the driver did not fill in the biSizeImage field, then compute it
	// Each scan line of the image is aligned on a DWORD (32bit) boundary
	if (bi.biSizeImage == 0){
		bi.biSizeImage = ((((bi.biWidth * bi.biBitCount) + 31) & ~31) / 8) 
						* bi.biHeight;

		// If a compression scheme is used the result may infact be larger
		// Increase the size to account for this.
		if (dwCompression != BI_RGB)
			bi.biSizeImage = (bi.biSizeImage * 3) / 2;
	}

	// Realloc the buffer so that it can hold all the bits
	dwLen += bi.biSizeImage;
	if (handle = GlobalReAlloc(hDIB, dwLen, GMEM_MOVEABLE))
		hDIB = handle;
	else{
		GlobalFree(hDIB);

		// Reselect the original palette
		SelectPalette(hDC,hPal,FALSE);
		ReleaseDC(NULL,hDC);
		return NULL;
	}

	// Get the bitmap bits
	lpbi = (LPBITMAPINFOHEADER)hDIB;

	// FINALLY get the DIB
	BOOL bGotBits = GetDIBits( hDC, (HBITMAP)bitmap.GetSafeHandle(),
				0L,				// Start scan line
				(DWORD)bi.biHeight,		// # of scan lines
				(LPBYTE)lpbi 			// address for bitmap bits
				+ (bi.biSize + nColors * sizeof(RGBQUAD)),
				(LPBITMAPINFO)lpbi,		// address of bitmapinfo
				(DWORD)DIB_RGB_COLORS);		// Use RGB for color table

	if( !bGotBits )
	{
		GlobalFree(hDIB);
		
		SelectPalette(hDC,hPal,FALSE);
		ReleaseDC(NULL,hDC);
		return NULL;
	}

	SelectPalette(hDC,hPal,FALSE);
	ReleaseDC(NULL,hDC);
	return hDIB;
}

// write the formula into a metafile (strFile)
BOOL CBinTree::WriteWMF(CString strFile) 
{
	CDC DC;
	CDC* pDC = NULL;
	CMetaFileDC MetaDC;

	HENHMETAFILE hEnhMetaFile;
	
	DC.CreateCompatibleDC(NULL);

	// Get the extent of the formula
	CRect rectMeta(0,0,0,0);
	CRect rect=GetRect(&DC);
	rectMeta.right  = MulDiv(rect.Width()*100, DC.GetDeviceCaps(HORZSIZE), 
                                       DC.GetDeviceCaps(HORZRES));
	rectMeta.bottom = MulDiv(rect.Height()*100, DC.GetDeviceCaps(VERTSIZE),
                                       DC.GetDeviceCaps(VERTRES));

	// Create an enhanced Metafile DC
	MetaDC.CreateEnhanced(&DC, NULL, rectMeta, _T("Formula\0Image\0\0"));
	MetaDC.SetAttribDC(DC.m_hDC);
	MetaDC.SetMapMode(MM_TEXT);
	pDC = &MetaDC;

	CBrush brush;
	brush.CreateSysColorBrush(COLOR_WINDOW);
	pDC->FillRect(rect, &brush);
	brush.DeleteObject();

	DrawTree(pDC, rect);

	hEnhMetaFile=MetaDC.CloseEnhanced();
	BOOL bRet = FALSE;
	HENHMETAFILE hMetaCopy = ::CopyEnhMetaFile(hEnhMetaFile, strFile); 
	if (hMetaCopy)
	{
		bRet = TRUE;
		DeleteEnhMetaFile(hMetaCopy);
	}

	DeleteEnhMetaFile(hEnhMetaFile);

	return bRet;
}

// write the formula into a file (strFile)
BOOL CBinTree::WriteDIB(CString strFile) 
{ 
	CBitmap bitmap;
	CBitmap* pOldBitmap;
    HANDLE hDib;
    
	CDC MemDC;
	MemDC.CreateCompatibleDC(NULL);

	CRect rect=GetRect(&MemDC);
	
	// If we use CreateCompatibleBitmap, the bit depth is not deterministic.
	HBITMAP hBitmap = ::CreateBitmap(rect.Width(), rect.Height(), 
			1, MemDC.GetDeviceCaps(BITSPIXEL), NULL);
	if (NULL == hBitmap)
		return FALSE;

	bitmap.Attach(hBitmap);
	
	pOldBitmap = MemDC.SelectObject(&bitmap);

	MemDC.SetWindowOrg(rect.left, rect.top);
	MemDC.FillSolidRect(rect, RGB(255,255,255));
	
	DrawTree(&MemDC, rect);
	
	BITMAPFILEHEADER hdr; 
	LPBITMAPINFOHEADER lpbi; 

	hDib=DDBToDIB(bitmap, BI_RGB, NULL);
	if (!hDib) 
		return FALSE; 

	CFile file; 
	if(!file.Open( strFile, CFile::modeWrite|CFile::modeCreate)) 
		return FALSE; 

	lpbi=(LPBITMAPINFOHEADER)GlobalLock(hDib); 

	int nColors = 1 << lpbi->biBitCount; 
	if(nColors > 256) 
		nColors = 0; 

	// Fill in the fields of the file header 
	hdr.bfType = ((WORD) ('M' << 8) | 'B'); // is always "BM" 
	hdr.bfSize = GlobalSize (hDib) + sizeof( hdr ); 
	hdr.bfReserved1 = 0; 
	hdr.bfReserved2 = 0; 
	hdr.bfOffBits = (DWORD) (sizeof( hdr ) + lpbi->biSize + nColors * sizeof(RGBQUAD)); 

	// Write the file header 
	file.Write( &hdr, sizeof(hdr) ); 

	// Write the DIB header and the bits 
	file.Write( lpbi, GlobalSize(hDib) ); 
	GlobalUnlock(hDib); 
	MemDC.SelectObject(pOldBitmap);
	bitmap.DeleteObject();
	file.Close();

	return TRUE; 
}

// Print out the formula into DC
BOOL CBinTree::PrintDDB(CDC* pDC) 
{ 
	CBitmap bitmap;
	CBitmap* pOldBitmap;
    
	CDC MemDC;
	MemDC.CreateCompatibleDC(NULL);
	CRect rect=GetRect(&MemDC);
	
	// If we use CreateCompatibleBitmap, the bit depth is not deterministic.
	HBITMAP hBitmap = ::CreateBitmap(rect.Width(), rect.Height(), 
			1, MemDC.GetDeviceCaps(BITSPIXEL), NULL);
	if (NULL == hBitmap)
		return FALSE;

	bitmap.Attach(hBitmap);
	
	pOldBitmap = MemDC.SelectObject(&bitmap);

	MemDC.SetWindowOrg(rect.left, rect.top);
	MemDC.FillSolidRect(rect, RGB(255,255,255));
	
	DrawTree(&MemDC, rect);
	
	int nScrXLogPPI = MemDC.GetDeviceCaps(LOGPIXELSX);
	int nScrYLogPPI = MemDC.GetDeviceCaps(LOGPIXELSY);
	int nXLogPPI = pDC->GetDeviceCaps(LOGPIXELSX);
	int	nYLogPPI = pDC->GetDeviceCaps(LOGPIXELSY);
	double dRatioX = (double)nXLogPPI / (double)nScrXLogPPI;
	double dRatioY = (double)nYLogPPI / (double)nScrYLogPPI;

	pDC->StretchBlt(rect.left, 
		rect.top, 
		(int)(rect.Width()*dRatioX), 
		(int)(rect.Height()* dRatioY),
		&MemDC, 
		rect.left,
		rect.top, 
		rect.Width(), 
		rect.Height(),
		SRCCOPY);
	
	MemDC.SelectObject(pOldBitmap);
	bitmap.DeleteObject();

	return TRUE;
}

// sorts the tree, so that nodes are in the right precedence order 
// uses GetAssociativityLevel of the node to determine the associativity
// ^, *, /, +, - an so on
// not used yet!
void CBinTree::SortPreOrder(CNode* pNode)
{
	if(pNode->GetLeftChild()!=NULL)
	{
		if(pNode->GetAssociativityLevel() <  pNode->GetLeftChild()->GetAssociativityLevel() )
		{
			SwapLeftChildNode(pNode, pNode->GetLeftChild() );
		}
	}
	if(pNode->GetRightChild()!=NULL)
	{
		if(pNode->GetAssociativityLevel() <  pNode->GetRightChild()->GetAssociativityLevel() )
		{
			SwapRightChildNode(pNode, pNode->GetRightChild() );
		}
	}
	if(pNode->GetLeftChild()!=NULL)
		SortPreOrder(pNode->GetLeftChild());
	if(pNode->GetRightChild()!=NULL)
		SortPreOrder(pNode->GetRightChild());
}

// exchanges nodes
CNode* CBinTree::SwapLeftChildNode(CNode* pNewNode, CNode* pOldNode)
{
	pNewNode->SetLeftChild(pOldNode->GetRightChild());
	pNewNode->GetParentDirect()->SetRightChild(pOldNode);
	pOldNode->SetRightChild(pNewNode);

	return pNewNode;
}

// exchanges nodes
CNode* CBinTree::SwapRightChildNode(CNode* pNewNode, CNode* pOldNode)
{
	pNewNode->SetRightChild(pOldNode->GetRightChild());
	pNewNode->GetParentDirect()->SetRightChild(pOldNode);
	pOldNode->SetRightChild(pNewNode);

	return pNewNode;
}

// Generate the Textoutput of the formula 
// nLanguageType is one of the LT_* defines
CString CBinTree::WriteFormula(int nLanguageType)
{
	CString strReturn=_T("      ");

	if(m_pRootNode)
		strReturn+=m_pRootNode->WriteNode(nLanguageType);

	if(nLanguageType==LT_FORTRAN)
	{
		if(strReturn.GetLength()>60)
		{
			int nCurPos=60;
			while(nCurPos < strReturn.GetLength())
			{
				strReturn.Insert(nCurPos, "\r\n     +");
				nCurPos+=67;
			}
		}
	}

	return strReturn;
}
