/////////////////////////////////////////////////////////////////////////////
//  File:       FormulaDlg_ToolBar.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//	FormulaDlg.cpp : implementation file
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
// Version 1.0.0.1: Added switching from constant to variable
//					suggested by John A. Johnson
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "mathsym.h"
#include "FormulaDlg.h"
#include "FormulaCtrl.h"
#include "MatrixDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFormulaDlg dialog

LONG CFormulaDlg::ToolBarCtrlSelected(WPARAM wParam , LPARAM lParam)
{
	CString strTemp;
	CNode* pNode=NULL;
	CNode* pParent=NULL;
	CNode* pNewNode=NULL;
	CString strType;

	if(wParam==-1)
		return 0L;

	switch (lParam)
	{
	case IDC_EQUATION: // from Toolbar Equations
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			// if  root node selected
			if(pNode==GetBinTree()->GetRootNode())
			{
				// switch the type of the node
				((CEquationNode*)pNode)->SetType(wParam);
				m_FormulaCtrl.Invalidate(TRUE);
				// set a checkpoint for undo/redo
				GetBinTree()->CheckPoint();
				break;
			}
		}
		// reset the tree
		GetBinTree()->ResetTree();
		// create a new formula
		pNewNode=GetBinTree()->CreateTree(NT_EQUATION);
		if(pNewNode)
		{
			// set left and right child as placeholder
			((CEquationNode*)pNewNode)->SetType(wParam);
			pNewNode->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
			pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_BRACE: // form toolbar Braces
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a brace node as pParents new child and let pNode sink down
				// as pNewNodes left child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						NT_BRACE, 
						NI_CHILDLEFT);
				// set the brace type
				((CBraceNode*)pNewNode)->SetType(wParam);
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_COREDATA: // form toolbar Core Data
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			// if the seleceted node is a leaf
			if(pNode->IsNodeLeaf() || 
				pNode->GetNodeType()==NT_NUMBER || 
				pNode->GetNodeType()==NT_CONSTANT || 
				pNode->GetNodeType()==NT_VARIABLE || 
				pNode->GetNodeType()==NT_INFINITY ||
				pNode->GetNodeType()==NT_PLACEHOLDER)
			{
				// replace the selected node with the new node
				if(wParam== 0)
				{
					pNewNode=GetBinTree()->ReplaceNode(pNode, NT_NUMBER);
				}
				if(wParam== 1)
				{
					if(pNode->GetNodeType()==NT_VARIABLE)
					{
						CString strShortCut=pNode->GetShortCut();
						CString strName=pNode->GetName();
						CString strFaceName=pNode->GetFaceName();
						double dValue=((CValueNode*)pNode)->GetValue();
						long lContext=((CValueNode*)pNode)->GetContext();
						long lUnit=((CValueNode*)pNode)->GetUnit();
						
						pNewNode=GetBinTree()->ReplaceNode(pNode, NT_CONSTANT);

						pNewNode->SetShortCut(strShortCut);
						pNewNode->SetName(strName);
						pNewNode->SetFaceName(strFaceName);
						((CValueNode*)pNewNode)->SetValue(dValue);
						((CValueNode*)pNewNode)->SetContext(lContext);
						((CValueNode*)pNewNode)->SetUnit(lUnit);
					}
					else	
						pNewNode=GetBinTree()->ReplaceNode(pNode, NT_CONSTANT);
				}
				if(wParam== 2)
				{
					if(pNode->GetNodeType()==NT_CONSTANT)
					{
						CString strShortCut=pNode->GetShortCut();
						CString strName=pNode->GetName();
						CString strFaceName=pNode->GetFaceName();
						double dValue=((CValueNode*)pNode)->GetValue();
						long lContext=((CValueNode*)pNode)->GetContext();
						long lUnit=((CValueNode*)pNode)->GetUnit();
						
						pNewNode=GetBinTree()->ReplaceNode(pNode, NT_VARIABLE);

						pNewNode->SetShortCut(strShortCut);
						pNewNode->SetName(strName);
						pNewNode->SetFaceName(strFaceName);
						((CValueNode*)pNewNode)->SetValue(dValue);
						((CValueNode*)pNewNode)->SetContext(lContext);
						((CValueNode*)pNewNode)->SetUnit(lUnit);
					}
					else	
						pNewNode=GetBinTree()->ReplaceNode(pNode, NT_VARIABLE);
				}
				if(wParam== 3)
				{
					pNewNode=GetBinTree()->ReplaceNode(pNode, NT_INFINITY);
				}
				if(wParam== 4)
				{
					pNewNode=GetBinTree()->ReplaceNode(pNode, NT_PLANCK);
				}
				if(wParam== 5)
				{
					pNewNode=GetBinTree()->ReplaceNode(pNode, NT_LAMBDA);
				}
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_SUM: // form toolbar Sum
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a sum node as pParents new child and let pNode sink down
				// as pNewNodes right child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						NT_SUM, 
						NI_CHILDRIGHT);
				// pNewNodes left child is by Constructor a Range Node
				// Set upper range as Placeholder
				if (wParam==0)
					pNewNode->GetLeftChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				// Set lower range as equation
				if (wParam==0 || wParam==1)
				{
					pNewNode->GetLeftChild()->SetRightChild(GetBinTree()->CreateNode(NT_EQUATION));
					// Set lower range equations children as Placeholders
					pNewNode->GetLeftChild()->GetRightChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
					pNewNode->GetLeftChild()->GetRightChild()->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_PROD: // form toolbar Prod (same behaviour as sum)
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a product node as pParents new child and let pNode sink down
				// as pNewNodes right child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						NT_PROD, 
						NI_CHILDRIGHT);
				// pNewNodes left child is by Constructor a Range Node
				// Set upper range as Placeholder
				if (wParam==0)
					pNewNode->GetLeftChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				// Set lower range as equation
				if (wParam==0 || wParam==1)
				{
					pNewNode->GetLeftChild()->SetRightChild(GetBinTree()->CreateNode(NT_EQUATION));
					// Set lower range equations children as Placeholders
					pNewNode->GetLeftChild()->GetRightChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
					pNewNode->GetLeftChild()->GetRightChild()->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_FUNCTION: // form toolbar Function
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode)
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a limes node as pParents new child and let pNode sink down
				// as pNewNodes right child
				if(wParam==21)
				{
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_LIMES, 
							NI_CHILDRIGHT);
					// Set lower range as arrow  x->y
					pNewNode->SetLeftChild(GetBinTree()->CreateNode(NT_OPERATOR));
					// Set arrows children as Placeholders
					pNewNode->GetLeftChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
					pNewNode->GetLeftChild()->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
				else if(wParam==22)
				{
					// create the binomial node  
					// first insert a brace node as pParents new child and let 
					// pNode sink down as pNewNodes left child (the brace content)
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_BRACE, 
							NI_CHILDLEFT);
					// switch the type of the brace node
					((CBraceNode*)pNewNode)->SetType(0);
					// now insert a binomial node into the brace and set the old 
					// brace content as its left child
					pNewNode=GetBinTree()->InsertNode(
										pNewNode, 
										pNode, 
										NT_BINOMIAL, 
										NI_CHILDLEFT);
					// Set the binomial nodes right child as Placeholder
					pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
				else if(wParam==23)
				{
					// insert a standard function  node as pParents new child and 
					// let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_USERFUNC, 
							NI_CHILDRIGHT);
					// insert a brace node as the function nodes new child and let 
					// pNode sink down as pNewNodes left child
					pNewNode=GetBinTree()->InsertNode(
							pNewNode, 
							pNode, 
							NT_BRACE, 
							NI_CHILDLEFT);
					// switch the type of the brace node
					((CBraceNode*)pNewNode)->SetType(0);
					// strip the NE_ALLOWDELETE flag
					pNewNode->SetEditMode(pNewNode->GetEditMode() & ~NE_ALLOWDELETE);
				}
				else
				{
					// insert a standard function  node as pParents new child and 
					// let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_FUNC, 
							NI_CHILDRIGHT);
					// switch the type of the node
					((CFuncNode*)pNewNode)->SetType(wParam);

					// insert a brace node as the function nodes new child and let 
					// pNode sink down as pNewNodes left child
					pNewNode=GetBinTree()->InsertNode(
							pNewNode, 
							pNode, 
							NT_BRACE, 
							NI_CHILDLEFT);
					// switch the type of the brace node
					((CBraceNode*)pNewNode)->SetType(0);
					// strip the NE_ALLOWDELETE flag
					pNewNode->SetEditMode(pNewNode->GetEditMode() & ~NE_ALLOWDELETE);
				}
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_INTEGRAL: // form toolbar Integral
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a integral node as pParents new child and let pNode sink down
				// as pNewNodes right child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						NT_INTEGRAL, 
						NI_CHILDRIGHT);
				// switch the type of the node
				((CIntegralNode*)pNewNode)->SetType(wParam);

				// pNewNodes left child is by Constructor a Range Node
				// Set lower range as Placeholder
				if (wParam==1 || wParam==2 || wParam==4 || wParam==5 || wParam==7 ||wParam==8)
				{
					pNewNode->GetLeftChild()->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
					// Set upper range as Placeholder 
					if (wParam==2 || wParam==5 || wParam==8)
						pNewNode->GetLeftChild()->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
				// insert a integrand node as pNewNodes new child and let pNode sink down
				// as pNewNodes left child
				pNewNode=GetBinTree()->InsertNode(
						pNewNode, 
						pNode, 
						NT_INTEGRAND, 
						NI_CHILDLEFT);
				//Set the right child of the integrand node as a Placeholder
				pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_DERIVATIVE: // form toolbar derivative
		// detect the type of the new node, dependent on the toolbar selection
		strType=NT_DERIVE;
		if(wParam== 0)
			strType=NT_DERIVE;
		if(wParam== 1)
			strType=NT_PARTDERIVE;
		if(wParam== 2)
			strType=NT_NABLA;
		if(wParam== 3)
			strType=NT_NABLA;
		if(wParam== 4)
			strType=NT_NABLA;
		if(wParam== 5)
			strType=NT_LAPLACE;

		pNode=GetBinTree()->GetSelectedNode();
		if(pNode)//if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				if(wParam<2)
				{
					// insert a derivative or partial derivative node as pParents 
					// new child and let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							strType, 
							NI_CHILDRIGHT);
					// Set derivative var. as Placeholder
					pNewNode->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
				}
				if(wParam==2)
				{
					// insert a cross node as pParents 
					// new child and let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_CROSS, 
							NI_CHILDRIGHT);
					// Set left child of pNewNode as the nabla operator
					pNewNode->SetLeftChild(GetBinTree()->CreateNode(strType));
				}
				if(wParam==3)
				{
					// insert a times node as pParents 
					// new child and let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							NT_TIMES, 
							NI_CHILDRIGHT);
					// Set left child of pNewNode as the nabla operator
					pNewNode->SetLeftChild(GetBinTree()->CreateNode(strType));
				}
				if(wParam>=4)
				{
					// insert a the nabla node node as pParents 
					// new child and let pNode sink down as pNewNodes right child
					pNewNode=GetBinTree()->InsertNode(
							pParent, 
							pNode, 
							strType, 
							NI_CHILDRIGHT);
				}
			}
			/*
			// no leaf selected and selectiontype is not NS_SUBTREE
			else
				// simply replace the selected node with a new node
				// warning: wont work very well, too many exceptions are possible
				GetBinTree()->ReplaceNode(pNode, strType);
			*/
		}

		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_PBYQ: // form toolbar Operators
		// detect the type of the new node, dependent on the toolbar selection
		strType=NT_DIVISION;
		if(wParam== 0 || wParam== 1)
			strType=NT_DIVISION;
		if(wParam== 2)
			strType=NT_CROSS;
		if(wParam== 3)
			strType=NT_TIMES;
		if(wParam== 4)
			strType=NT_PLUS;
		if(wParam== 5)
			strType=NT_MINUS;
		if(wParam== 6)
			strType=NT_ROOT;
		if(wParam== 7)
			strType=NT_NTHROOT;
		if(wParam== 8)
			strType=NT_POWERTO;

		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert the new node as pParents new child and let pNode sink down
				// as pNewNodes left child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						strType, 
						NI_CHILDLEFT);
				// if a binary operator is selected (root is unitary) set 
				// the right child as Placeholder
				if(strType!=NT_ROOT)
					pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
			}
			// no leaf selected and selectiontype is not NS_SUBTREE
			/*else
				// simply replace the selected node with a new node
				// warning: wont work very well, too many exceptions are possible
				GetBinTree()->ReplaceNode(pNode, strType);
			*/
			// if node is NT_DIVISION, then switch to the right type
			if(strType==NT_DIVISION && pNewNode!=NULL)
				((CDivisionNode*)pNewNode)->SetType(wParam);
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_MATRIX: // form toolbar matrix
		{
			strType=NT_MATRIX;
			CNode* pLineNode=NULL;
			CNode* pMatrixNode=NULL;
			CNode* pElementNode=NULL;
			int n=-1;
			int	m=-1;
			// get the dimension of the matrix (n rows, m columns)
			if(wParam== 0)
			{	// 1,2
				n=1;
				m=2;
			}
			if(wParam== 1)
			{	// 2,1
				n=2;
				m=1;
			}
			if(wParam== 2)
			{	// 2,2
				n=2;
				m=2;
			}
			if(wParam== 3)
			{	// 1,3
				n=1;
				m=3;
			}
			if(wParam== 4)
			{	// 3,1
				n=3;
				m=1;
			}
			if(wParam== 5)
			{	// 3,3
				n=3;
				m=3;
			}
			if(wParam== 6)
			{	// 1,4
				n=1;
				m=4;
			}
			if(wParam== 7)
			{	// 4,1
				n=4;
				m=1;
			}
			if(wParam== 8)
			{	// 4,4
				n=4;
				m=4;
			}
			if(wParam== 9)
			{	// 1,m
				CMatrixDlg dlg;
				dlg.m_nRowCount=-1;
				if(dlg.DoModal()== IDOK)
				{
					n=1;
					m=dlg.m_nColumnCount;
				}
				else
					return 0L;
			}
			if(wParam== 10)
			{	// n,1
				CMatrixDlg dlg;
				dlg.m_nColumnCount=-1;
				if(dlg.DoModal()== IDOK)
				{
					m=1;
					n=dlg.m_nRowCount;
				}
				else
					return 0L;
			}
			if(wParam== 11)
			{	// n,m
				CMatrixDlg dlg;
				if(dlg.DoModal()== IDOK)
				{
					n=dlg.m_nRowCount;
					m=dlg.m_nColumnCount;
				}
				else
					return 0L;
			}

			pNode=GetBinTree()->GetSelectedNode();
			if(pNode) //if any node selected 
			{
				//if selected node is root then break
				if(pNode==GetBinTree()->GetRootNode())
					break;
				// get the parent node (it must be vaild because pNode is not root)
				pParent=GetBinTree()->GetParent(pNode);
				// if the selectiontype is NS_SUBTREE or pNode is a leaf
				if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
				{
					// insert a brace node as pParents 
					// new child and let pNode sink down as pNewNodes left child
					pNewNode=GetBinTree()->InsertNode(
						pParent,
						pNode,
						NT_BRACE,
						NI_CHILDLEFT);
					// set the brace type
					((CBraceNode*)pNewNode)->SetType(1);
					// create a Matrix-node
					pMatrixNode=GetBinTree()->CreateNode(NT_MATRIX);
					// set the matrix node as the brace nodes left child
					pNewNode->SetLeftChild(pMatrixNode);
					// start form the matrix node and append the Lines
					for (int nLine=0; nLine<n; nLine++)
					{
						pLineNode=GetBinTree()->CreateNode(NT_LINE);
						// create the new line as the left child of the previous line
						// or in the beginning of the matrix node
						pMatrixNode->SetLeftChild(pLineNode);
						// start form the line node and append the Elements (columns)
						for (int nElement=0; nElement<m; nElement++)
						{
							// create the new element as the right child of the previous 
							pElementNode=GetBinTree()->CreateNode(NT_ELEMENT);
							// element or in the beginning of the line node
							pLineNode->SetRightChild(pElementNode);
							if(nElement==0 && nLine==0)
								// set the slected node into top left position
								pElementNode->SetLeftChild(pNode);
							else
								// all other elements are placeholder
								pElementNode->SetLeftChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
	
							pLineNode=pLineNode->GetRightChild();
						} // next element
						pMatrixNode=pMatrixNode->GetLeftChild();
					}// next line
				}
				/*
				// no leaf selected and selectiontype is not NS_SUBTREE
				else
					// simply replace the selected node with a new node
					// warning: wont work very well, too many exceptions are possible
					GetBinTree()->ReplaceNode(pNode, strType);
				*/
			}
			m_FormulaCtrl.Invalidate(TRUE);
			break;
		}
	case IDC_VECTOR: // form toolbar vector
		strType=NT_VECTOR;
		// if wParam < 7 then a index node (sub superscript) is selected
		if(wParam> 7)
			strType=NT_INDEX;

		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode==GetBinTree()->GetRootNode())
				break;
			if(strType==NT_INDEX) 
			{
				// only insert sub superscripts if the node is a leaf
				if(pNode->IsNodeLeaf())
				{
					// create a index Node (strType is set in the beginning)
					pNewNode=GetBinTree()->CreateNode(strType);
					// set the sub superscript as pnodes new right child
					pNode->SetRightChild(pNewNode);
					// set the superscript node as Placeholder
					if(wParam==8 || wParam==9)
						pNewNode->SetLeftChild(GetBinTree()->CreateNode(NT_LABEL));
					// set the subscript node as Placeholder
					if(wParam==8 || wParam==10)
						pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_LABEL));
				}
				m_FormulaCtrl.Invalidate(TRUE);
				break;
			}
			
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert the vector node as pParents 
				// new child and let pNode sink down as pNewNodes left child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						strType, 
						NI_CHILDLEFT);
			}
			// if node is a vector node, then switch to the right type
			if(strType==NT_VECTOR && pNewNode)
				((CVectorNode*)pNewNode)->SetType(wParam);
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_ARROWS: // form toolbar arrows
		strType=NT_ARROWS;
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode == GetBinTree()->GetRootNode())
				break;
			// get the parent node (it must be vaild because pNode is not root)
			pParent=GetBinTree()->GetParent(pNode);
			// if the selectiontype is NS_SUBTREE or pNode is a leaf
			if(GetBinTree()->GetNodeSelectionType()==NS_SUBTREE || pNode->IsNodeLeaf())
			{
				// insert a arrow node as pParents new child and let pNode sink down
				// as pNewNodes left child
				pNewNode=GetBinTree()->InsertNode(
						pParent, 
						pNode, 
						strType, 
						NI_CHILDLEFT);
				// switch the type of the arrow node
				((CArrowNode*)pNewNode)->SetType(wParam);
				// set the right child as Placeholder
				pNewNode->SetRightChild(GetBinTree()->CreateNode(NT_PLACEHOLDER));
			}
			// no leaf selected and selectiontype is not NS_SUBTREE
			/*else
			{
				// simply replace the selected node with a new node
				// warning: wont work very well, too many exceptions are possible
				pNewNode=GetBinTree()->ReplaceNode(pNode, strType);
				((CArrowNode*)pNewNode)->SetType(wParam);
			}*/
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_UPPER_GREEK: // form toolbar upper greek
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode == GetBinTree()->GetRootNode())
				break;
			// get the type of the selected node
			strType=pNode->GetNodeType();
			// if pNode is a placeholder , replace it with a variable node
			if (strType== NT_PLACEHOLDER)
			{
				pNewNode=GetBinTree()->ReplaceNode(pNode, NT_VARIABLE);
				pNode=pNewNode;
				strType=pNode->GetNodeType();
			}
			// if pNode is a variable or constant, set the right shortcut
			if (strType== NT_VARIABLE || strType== NT_CONSTANT)
			{
				pNode->SetFaceName("Symbol");
				pNode->GetLogFont()->lfCharSet=SYMBOL_CHARSET;
				strTemp.Format("%c",'A'+wParam);
				pNode->SetShortCut(strTemp);
				// set a checkpoint for undo/redo
				if(!pNewNode)
					GetBinTree()->CheckPoint();
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;

	case IDC_LOWER_GREEK: // form toolbar lower greek (same behaviour as upper greek)
		pNode=GetBinTree()->GetSelectedNode();
		if(pNode) //if any node selected 
		{
			//if selected node is root then break
			if(pNode == GetBinTree()->GetRootNode())
				break;
			// get the type of the selected node
			strType=pNode->GetNodeType();
			// if pNode is a placeholder , replace it with a variable node
			if (strType== NT_PLACEHOLDER)
			{
				pNewNode=GetBinTree()->ReplaceNode(pNode, NT_VARIABLE);
				pNode=pNewNode;
				strType=pNode->GetNodeType();
			}
			// if pNode is a variable or constant, set the right shortcut
			if (strType==NT_VARIABLE || strType==NT_CONSTANT)
			{
				pNode->SetFaceName("Symbol");
				pNode->GetLogFont()->lfCharSet=SYMBOL_CHARSET;
				strTemp.Format("%c",'a'+wParam);
				pNode->SetShortCut(strTemp);
				// set a checkpoint for undo/redo
				GetBinTree()->CheckPoint();
			}
		}
		m_FormulaCtrl.Invalidate(TRUE);
		break;
		
		default:
			break;
	}

	// set a checkpoint for undo/redo
	if(pNewNode!=NULL)
		GetBinTree()->CheckPoint();

    return 1L;
}
