/////////////////////////////////////////////////////////////////////////////
//  File:       node.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "node.h"
#include "bintree.h"

void CenterRectVert(CRect& rectSource, CRect& rectDest)
{
	int nOffset=(rectDest.Height()-rectSource.Height())/2;
	rectSource.top+=nOffset;
	rectSource.bottom+=nOffset;
}

void CenterRectHorz(CRect& rectSource, CRect& rectDest)
{
	int nOffset=(rectDest.Width()-rectSource.Width())/2;
	rectSource.left+=nOffset;
	rectSource.right+=nOffset;
}

//=======================================================
IMPLEMENT_DYNAMIC( CNode, CObject)

CNode::CNode()
{
	m_strName = "";
	m_strShortCut= "";
	m_pParent=NULL;
	m_pLeftChild = NULL;
	m_pRightChild = NULL;
	m_NodeRect=CRect(0,0,0,0);
	m_nBaseLine=0;

	m_dwEditMode= NE_ALLOWDELETE;

	m_nSubLevel=0;
	m_nSubLevelRightInc=0;
	m_nSubLevelLeftInc=0;
	
	m_nAssociativityLevel=-1;
	m_strNodeType="";
	m_crColor=::GetSysColor(COLOR_WINDOWTEXT);

   	m_lf.lfHeight=GetFormat()->GetFontHeight(FH_STANDARD)*10;//100;
   	m_lf.lfWidth=0;
   	m_lf.lfEscapement=0;
   	m_lf.lfOrientation=0;
   	m_lf.lfWeight=FW_NORMAL;
   	m_lf.lfItalic=TRUE;
	m_lf.lfUnderline=FALSE;
	m_lf.lfStrikeOut=FALSE;
	m_lf.lfCharSet=ANSI_CHARSET;
	m_lf.lfOutPrecision=OUT_CHARACTER_PRECIS;
	m_lf.lfClipPrecision=CLIP_CHARACTER_PRECIS;
	m_lf.lfQuality= ANTIALIASED_QUALITY;
	m_lf.lfPitchAndFamily=FF_DECORATIVE;
	strcpy(m_lf.lfFaceName, "Times New Roman");

	for (int i=0; i<LT_COUNT; i++)
		m_strCodeFormat[i]=_T("%s %s %s");
}

CNode::~CNode()
{
	if(m_pLeftChild)
		delete m_pLeftChild;
	if(m_pRightChild)
		delete m_pRightChild;
}

CNode* CNode::FindParentPraeorder(CNode* pSearchNode)
{
	CNode* pResultNode=NULL;
	if(m_pLeftChild==pSearchNode || m_pRightChild==pSearchNode)
		return this;
	if(m_pLeftChild)
	{
		pResultNode=m_pLeftChild->FindParentPraeorder(pSearchNode);
		if(pResultNode)
			return pResultNode;
	}

	if(m_pRightChild)
		pResultNode=m_pRightChild->FindParentPraeorder(pSearchNode);

	return pResultNode;

}

BOOL CNode::IsNodeLeaf()
{
	if(GetLeftChild() || GetRightChild())
		return FALSE;
	else 
		return TRUE;
}

void CNode::SetEditMode(DWORD dwEditMode)
{
	m_dwEditMode=dwEditMode;
}

DWORD CNode::GetEditMode()
{
	return m_dwEditMode;
}

LOGFONT* CNode::GetLogFont()
{
	return &m_lf;
}
CString CNode::GetNodeType()
{
	return m_strNodeType;
}

void CNode::SetLeftChild(CNode* pNode)
{
	m_pLeftChild=pNode;
	if(m_pLeftChild)
	{
		m_pLeftChild->m_pParent=this;
		m_pLeftChild->m_nSubLevel=m_nSubLevel+m_nSubLevelLeftInc; // inherit the SubLevel
		m_pLeftChild->FormatNode(GetFormat());
	}
}

CNode* CNode::GetLeftChild()
{
	return m_pLeftChild;
}

void CNode::SetRightChild(CNode* pNode)
{
	m_pRightChild=pNode;
	if(m_pRightChild)
	{
		m_pRightChild->m_pParent=this;
		m_pRightChild->m_nSubLevel=m_nSubLevel+m_nSubLevelRightInc; // inherit the SubLevel
		m_pRightChild->FormatNode(GetFormat());
	}
}

CNode* CNode::GetParentDirect()
{
	return m_pParent;
}

void CNode::SetParentDirect(CNode* pNode)
{
	m_pParent=pNode;
}

CNode* CNode::GetRightChild()
{
	return m_pRightChild;
}

void CNode::SetName(CString strName)
{
	m_strName=strName;
}

CString CNode::GetName()
{
	return m_strName;
}

void CNode::SetShortCut(CString strShortCut)
{
	m_strShortCut=strShortCut;
}

CString CNode::GetShortCut()
{
	return m_strShortCut;
}

void CNode::SetItalic(BOOL bItalic)
{
	m_lf.lfItalic=bItalic==TRUE ? 1: 0;
}

BOOL CNode::GetItalic()
{
	return (m_lf.lfItalic==1 ? TRUE: FALSE);
}
	
void CNode::SetBold(BOOL bBold)
{
	m_lf.lfWeight= (bBold==TRUE) ? FW_BOLD : FW_NORMAL;
}

BOOL CNode::GetBold()
{
	return  (m_lf.lfWeight==FW_BOLD ? TRUE : FALSE);
}

void CNode::SetColor(COLORREF crColor)
{
	m_crColor=crColor;
}

COLORREF CNode::GetColor()
{
	return  m_crColor;
}

void CNode::SetSubLevel(int nSubLevel)
{
	m_nSubLevel=nSubLevel;
}

int CNode::GetSubLevel()
{
	return  m_nSubLevel;
}

int CNode::GetSubLevelRightInc()
{
	return  m_nSubLevelRightInc;
}

int CNode::GetSubLevelLeftInc()
{
	return  m_nSubLevelLeftInc;
}

void CNode::SetFaceName(CString strFaceName)
{
	strcpy(m_lf.lfFaceName, (LPCSTR)strFaceName);
}

CString CNode::GetFaceName()
{
	return m_lf.lfFaceName;
}

BOOL CNode::operator== (const CNode& Node)
{
	return (this->m_strName == Node.m_strName);
}

const void CNode::operator=(const CNode& Node)
{
    CNode& rNode = (CNode&) Node;

// copy the content of this node
	m_nAssociativityLevel=	rNode.GetAssociativityLevel ();
	m_dwEditMode=			rNode.GetEditMode();
	m_nSubLevel=			rNode.GetSubLevel();
	m_nSubLevelLeftInc=		rNode.GetSubLevelLeftInc();
	m_nSubLevelRightInc=	rNode.GetSubLevelRightInc();
	m_strNodeType=			rNode.GetNodeType();
	m_strShortCut=			rNode.GetShortCut();
	m_strName=				rNode.GetName();
	m_crColor=				rNode.GetColor();
	m_NodeRect=				rNode.m_NodeRect;
	m_nBaseLine=			rNode.m_nBaseLine;

	memcpy(GetLogFont(), rNode.GetLogFont(), sizeof(LOGFONT));
}

// default-behaviour (simply write out the text into the rectangle rect)
void CNode::DrawContent(CDC* pDC, CRect rect, DWORD dwSelect)
{
	CFont ctrlFont;

	ctrlFont.CreatePointFontIndirect
	(
		&m_lf,
		pDC
	);

	CFont* pOldFont=pDC->SelectObject(&ctrlFont);

	pDC->DrawText(m_strShortCut, rect, DT_NOCLIP | DT_NOPREFIX | DT_SINGLELINE | DT_CENTER | DT_VCENTER);

	pDC->SelectObject(pOldFont);
	ctrlFont.DeleteObject();
}

// draw the node pre-order 
// first myself
// second the left child 
// third the right child
void CNode::DrawNode(CDC* pDC, CRect& rect, CNode* pSelectNode, DWORD dwSelect)
{
// empty rects
	CRect contentRect(0,0,0,0);
	CRect leftRect(0,0,0,0);
	CRect rightRect(0,0,0,0);

// calculate the destination rectangles
	if(m_pLeftChild)
	{
		leftRect=m_pLeftChild->GetRect(pDC);
	}
	
	contentRect=GetContentRect(pDC);

	if(m_pRightChild)
	{
		rightRect=m_pRightChild->GetRect(pDC);
	}

	TransformRects(rect, leftRect, rightRect, contentRect);
	m_NodeRect=contentRect;

// Draw myself only if node is not in editmode, 
// after setting up devices
	if( !(m_dwEditMode & NE_INEDITMODE))
	{

		CPen pen;
		CPen* pOldPen;

		if(dwSelect & NS_DISABLED)
		{
			pDC->FillSolidRect(contentRect, ::GetSysColor(COLOR_WINDOW));
			pen.CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_GRAYTEXT));
			pOldPen=pDC->SelectObject(&pen);

			pDC->SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
			pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));

			DrawContent(pDC, contentRect, dwSelect);
		}
		else
		{
			if(this==pSelectNode)
			{
				pDC->FillSolidRect(contentRect, ::GetSysColor(COLOR_HIGHLIGHT));
				pen.CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_HIGHLIGHTTEXT));
				pOldPen=pDC->SelectObject(&pen);

				pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
				pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));

				DrawContent(pDC, contentRect, dwSelect);
			}
			else
			{
				pDC->FillSolidRect(contentRect, ::GetSysColor(COLOR_WINDOW));
				pen.CreatePen(PS_SOLID, 1, m_crColor);
				pOldPen=pDC->SelectObject(&pen);

				pDC->SetTextColor(m_crColor);
				pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));

				DrawContent(pDC, contentRect, NULL);
			}
		}

// clean up devices
		pDC->SelectObject(pOldPen);
		pen.DeleteObject();
	}
	else
		TRACE("%s: %s\n", m_strNodeType, m_strShortCut);
	
// Draw left child
	if(m_pLeftChild)
	{
		if(this==pSelectNode && (dwSelect & NS_SUBTREE))
		{	
			m_pLeftChild->DrawNode(pDC, leftRect, m_pLeftChild, dwSelect);
		}
		else
		{
			m_pLeftChild->DrawNode(pDC, leftRect, pSelectNode, dwSelect);
		}
	}

// Draw right child
	if(m_pRightChild)
	{
		if(this==pSelectNode && (dwSelect & NS_SUBTREE))
		{
			m_pRightChild->DrawNode(pDC, rightRect, m_pRightChild, dwSelect);
		}
		else
		{
			m_pRightChild->DrawNode(pDC, rightRect, pSelectNode, dwSelect);
		}
	}
}

// default behaviour (calculate the extend of the content string)
CRect CNode::GetContentRect(CDC* pDC)
{
	CFont ctrlFont;

	ctrlFont.CreatePointFontIndirect
	(
		&m_lf,
		pDC
	);

	CFont* pOldFont=pDC->SelectObject(&ctrlFont);
	
	CSize sizeOut=pDC->GetOutputTextExtent(m_strShortCut);

	pDC->SelectObject(pOldFont);
	ctrlFont.DeleteObject();

	sizeOut+=CSize(4, 0);
	return CRect(CPoint(0,0), sizeOut);
}

// return the rectangle of the complete subtree
// transform the the rectangles of the left, the right child and of myself
// return the result-rect
CRect CNode::GetRect(CDC* pDC)
{
	CRect contentRect(0,0,0,0);
	CRect leftRect(0,0,0,0);
	CRect rightRect(0,0,0,0);

	if(m_pLeftChild)
		leftRect=m_pLeftChild->GetRect(pDC);
	
	contentRect=GetContentRect(pDC);

	if(m_pRightChild)
		rightRect=m_pRightChild->GetRect(pDC);
	
	CRect resultRect(0,0,0,0);
	
	TransformRects(resultRect, leftRect, rightRect, contentRect);

	return resultRect;
}

// default behaviour (left subtree on the left, right subtree on the 
// right and myself into the middle)
void CNode::TransformRects(CRect& rect,CRect& leftRect,CRect& rightRect,CRect& contentRect)
{
	int nTop= rect.top;
	int nLeft= rect.left;
	
	int nContentBaseLine=0;
	int nLeftBaseLine=0;
	int nRightBaseLine=0;
	int nOffset=0;

	nContentBaseLine=contentRect.Height()/2;
	nLeftBaseLine=nContentBaseLine;
	nRightBaseLine=nContentBaseLine;
	
	if(leftRect.Height() && m_pLeftChild!=NULL)
		nLeftBaseLine=m_pLeftChild->m_nBaseLine;
	if(rightRect.Height() && m_pRightChild!=NULL)
		nRightBaseLine=m_pRightChild->m_nBaseLine;
	
	nOffset=ternary_max(nContentBaseLine, nLeftBaseLine, nRightBaseLine);

	// adjusting baselines of subterms
	leftRect.OffsetRect(nLeft, nTop+nOffset-nLeftBaseLine);
	nLeft+=leftRect.Width()+1;

	contentRect.OffsetRect(nLeft, nTop+nOffset-nContentBaseLine);
	nLeft+=contentRect.Width()+1;

	rightRect.OffsetRect(nLeft, nTop+nOffset-nRightBaseLine);
	nLeft+=rightRect.Width()+1;

	rect.right=nLeft;
	rect.bottom=rect.top+ternary_max(
		leftRect.Height()-nLeftBaseLine+nOffset, 
		contentRect.Height()-nContentBaseLine+nOffset,
		rightRect.Height()-nRightBaseLine+nOffset);

	m_nBaseLine=nOffset;
}

// return the node of a given point (mouse coordinate)
CNode* CNode::GetNodeFromPoint(CDC* pDC, CRect& rect, CPoint point)
{
	CRect contentRect(0,0,0,0);
	CRect leftRect(0,0,0,0);
	CRect rightRect(0,0,0,0);
	CNode* pNode=NULL;
	if(m_pLeftChild)
		leftRect=m_pLeftChild->GetRect(pDC);
	
	contentRect=GetContentRect(pDC);

	if(m_pRightChild)
		rightRect=m_pRightChild->GetRect(pDC);

	TransformRects(rect, leftRect, rightRect, contentRect);

	if(leftRect.PtInRect(point))
		if(m_pLeftChild)
		{
			pNode=m_pLeftChild->GetNodeFromPoint(pDC, leftRect, point);
			rect=leftRect;
			if(pNode)
				return pNode;
		}

	if(rightRect.PtInRect(point))
		if(m_pRightChild)
		{
			pNode=m_pRightChild->GetNodeFromPoint(pDC, rightRect, point);
			rect=rightRect;
			if(pNode)
				return pNode;
		}
	
	if(contentRect.PtInRect(point))
	{
		rect=contentRect;
		return this;
	}

	return this;
}

// Set the Font properties of the node
void CNode::FormatNode(CBinTreeFormat* pFormat)
{
	// Adjust font size of the children and format them
	if(m_pLeftChild)
	{
		m_pLeftChild->m_nSubLevel=m_nSubLevel+m_nSubLevelLeftInc; // inherit the SubLevel
		m_pLeftChild->FormatNode(pFormat);
	}
	
	if(m_pRightChild)
	{
		m_pRightChild->m_nSubLevel=m_nSubLevel+m_nSubLevelRightInc; // inherit the SubLevel
		m_pRightChild->FormatNode(pFormat);
	}

	// Format myself
	SetFaceName(pFormat->m_FormatMap[m_strNodeType]->m_strFaceName);
	SetColor(pFormat->m_FormatMap[m_strNodeType]->m_crColor);
	SetBold(pFormat->m_FormatMap[m_strNodeType]->m_bBold);
	SetItalic(pFormat->m_FormatMap[m_strNodeType]->m_bItalic);
	m_lf.lfCharSet=pFormat->m_FormatMap[m_strNodeType]->m_lfCharSet;
	m_lf.lfHeight=pFormat->m_nFontHeight[m_nSubLevel > 2 ? FH_SUB_SUB : m_nSubLevel]*10;
}

// Serialize the node properties
// be careful, its recursive
void CNode::Serialize(CArchive& ar, CBinTree* pTree)
{
	CString strType;
	CNode* pLeftChild=NULL;
	CNode* pRightChild=NULL;
	
	if (ar.IsStoring())
	{
		if(m_pLeftChild)
		{
			strType=m_pLeftChild->m_strNodeType;
			ar << strType;
			m_pLeftChild->Serialize(ar, pTree);
		}
		else
		{
			strType="";
			ar << strType;
		}
	
		if(m_pRightChild)
		{
			strType=m_pRightChild->m_strNodeType;
			ar << strType;
			m_pRightChild->Serialize(ar, pTree);
		}
		else
		{
			strType="";
			ar << strType;
		}

		ar << m_dwEditMode;
		ar << m_nAssociativityLevel;
		ar << m_nSubLevel;
		ar << m_nSubLevelLeftInc;
		ar << m_nSubLevelRightInc;
		ar << m_strNodeType;
		ar << m_strShortCut;
		ar << m_strName;
		ar << m_crColor;
		ar << m_NodeRect;
		ar << m_nBaseLine;

		ar << GetFaceName();
		ar << GetItalic();
		ar << GetBold();
	}
	else
	{
		ar >> strType;
		if(strType!="")
		{
			if(m_pLeftChild)
				delete m_pLeftChild;
			pLeftChild=pTree->CreateNode(strType);
			pLeftChild->Serialize(ar, pTree);
		}
	
		ar >> strType;
		if(strType!="")
		{
			if(m_pRightChild)	
				delete m_pRightChild;
			pRightChild=pTree->CreateNode(strType);
			pRightChild->Serialize(ar, pTree);
		}

		ar >> m_dwEditMode;
		ar >> m_nAssociativityLevel;
		ar >> m_nSubLevel;
		ar >> m_nSubLevelLeftInc;
		ar >> m_nSubLevelRightInc;
		ar >> m_strNodeType;
		ar >> m_strShortCut;
		ar >> m_strName;
		ar >> m_crColor;
		ar >> m_NodeRect;
		ar >> m_nBaseLine;
		
		SetLeftChild(pLeftChild);
		SetRightChild(pRightChild);

		CString strFaceName;
		ar >> strFaceName;
		SetFaceName(strFaceName);
		
		BOOL bItalic;
		ar >> bItalic;
		SetItalic(bItalic);
	
		BOOL bBold;
		ar >> bBold;
		SetBold(bBold);
	}
}

// return the associativity of the node
// used for the precedence order  in bintree
// currently not used
int CNode::GetAssociativityLevel()
{
	return m_nAssociativityLevel;
}

// return the generated soucecode of the node
// generation is performed for the lanuage of type 
// nLanguageType (one of the LT_* defines)
// this function is used recursivly
CString CNode::WriteNode(int nLanguageType)
{
	ASSERT(nLanguageType>=0 && nLanguageType<LT_COUNT);
	
	CString strReturn=_T("");
	CString strLeft=_T("");
	CString strRight=_T("");
	CString strFormat=_T("");
	
	strFormat=m_strCodeFormat[nLanguageType];
	
	switch(nLanguageType)
	{
	case LT_FORTRAN:
	default:
		if(m_pLeftChild)
			strLeft=m_pLeftChild->WriteNode(nLanguageType);
		if(m_pRightChild)
			strRight=m_pRightChild->WriteNode(nLanguageType);
		strReturn.Format(strFormat, 
			strLeft, 
			m_strKeyWord[nLanguageType].IsEmpty() ? m_strShortCut : m_strKeyWord[nLanguageType], 
			strRight);
		break;
	}
	return strReturn;
}
